--
-- Rekordia
-- Specialization for Rekordia
--
-- @author Felix "Outlaw" Sorge
-- @date  20/05/09
-- @web www.ls-modsource.de - www.ls-mods.de
--
-- Copyright (C) Outlaw, All Rights Reserved.
--


Rekordia = {};

function Rekordia.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function Rekordia:load(xmlFile)

    self.setIsWaterTankFilling = Utils.overwrittenFunction(self.setIsWaterTankFilling, Rekordia.setIsWaterTankFilling);
	self.fertilizerInRange = SpecializationUtil.callSpecializationsFunction("fertilizerInRange");
	self.manureTanker = true;	
	self.literPerSecond = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.literPerSecond#value"), 1);
	self.manureTriggerCallback = Rekordia.manureTriggerCallback;
	self.trigger = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.Trigger#index"));	
	addTrigger(self.trigger, "manureTriggerCallback", self);
	self.deleteListenerId = addDeleteListener(self.trigger, "delete", self);
	
	self.chargeTypes = {};
	self.fillTypes[FruitUtil.FRUITTYPE_UNKNOWN] = true;
	local chargeTypes = getXMLString(xmlFile, "vehicle.chargeTypes#chargeTypes");
    if chargeTypes ~= nil then
		self.chargefillNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.chargeTypes#fillNode"));
        local types = Utils.splitString(" ", chargeTypes);
        for k,v in pairs(types) do
            local desc = FruitUtil.fruitTypes[v];
            if desc ~= nil then
                self.chargeTypes[desc.index] = true;
            end;
        end;
    end;
	
	
	local workSound = getXMLString(xmlFile, "vehicle.workSound#file");
    if workSound ~= nil and workSound ~= "" then
        workSound = Utils.getFilename(workSound, self.baseDirectory); 
        self.workSound = createSample("workSound");
        self.workSoundEnabled = false;
        loadSample(self.workSound, workSound, false);
    end;

	
	self.keyType = InputBinding.getKeyNamesOfDigitalAction;
	self.keyName = "Taste";	
	self.fill = false;
	self.fillSpeed = 1.00;
	self.trailerInTrigger = nil;	
	
end;


function Rekordia:update(dt)
							
	local manureFill = self.fill;
		if self.trailerInTrigger ~= nil and self.fillLevel > 0 then
		if self:getIsActiveForInput() then		
			if self.trailerInTrigger.fillLevel < self.trailerInTrigger.capacity then
				if InputBinding.hasEvent(InputBinding.pesticideTrailer_ON)  then
					self.fill = not self.fill; 
				end;
				if self.fill then
					g_currentMission:addExtraPrintText(g_i18n:getText("pesticideTrailer_off").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.pesticideTrailer_on)); 
				else
					g_currentMission:addExtraPrintText(g_i18n:getText("pesticideTrailer_on" ).." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.pesticideTrailer_off));
				end;
			end;
		end;
	end;
	if self.trailerInTrigger ~= nil and self.fillLevel > 0 then	
		if g_currentMission.controlledVehicle ~= nil then
			if self.trailerInTrigger.attacherVehicle == g_currentMission.controlledVehicle or self.trailerInTrigger == g_currentMission.controlledVehicle then
				if g_gui.currentGui == nil then
				end;
			end;
		end;
		local trailer = self.trailerInTrigger;
		if trailer.fillLevel < trailer.capacity then
			if g_server ~= nil then
				if self.fill then							
					local deltaFillLevel = dt*self.fillSpeed;
					trailer:setFillLevel(trailer.fillLevel+deltaFillLevel, self.currentFillType);
					self:setFillLevel(self.fillLevel-deltaFillLevel, self.currentFillType);
				end;
			end;
		else
			self.fill = true;
		end;
	else
		self.fill = true;
	end;
	if manureFill ~= self.fill then
		RekordiaEvent.sendEvent(self, self.fill);
	end;
	
   	self.inRange = false; 
    local nearestDistance = 10.0;
	local px, py, pz = getWorldTranslation(self.chargefillNode);
	for i=1, table.getn(g_currentMission.attachables) do
		local vx, vy, vz = getWorldTranslation(g_currentMission.attachables[i].rootNode);
		local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);

		if distance < nearestDistance then
			local attachable = g_currentMission.attachables[i];	
			if fillType == self.currentFillType and attachable.rootNode ~= self.rootNode and attachable.setFillLevel ~= nil then
				            
				if attachable.fillLevel == attachable.capacity  or self.filllevel == 0 then
					self.turnOn = false; 
					if self.workSoundEnabled then
						stopSample(self.workSound);
						self.workSoundEnabled = false;
					end;
				elseif attachable.fillLevel ~= attachable.capacity  and self.fillLevel ~= 0  then
					if self.turnOn then
						self.inRange = true; 
						local deltaLevel = self.literPerSecond*dt/1000.0;
						deltaLevel = math.min(deltaLevel, attachable.capacity - attachable.fillLevel);
						self.fillLevel = self.fillLevel-deltaLevel;
						self:setFillLevel(self.fillLevel, self.currentFillType);
						attachable:setFillLevel(attachable.fillLevel + deltaLevel, self.currentFillType);	
					end; 							
				end; 
			end;
		end;
    end;
		
	if not self.inRange and self.turnOn and self.workSoundEnabled then
		if self.workSoundEnabled then
			self.turnOn = not self.turnOn; 
			stopSample(self.workSound);
			self.workSoundEnabled = false;
		end;
	end;
end;


function Rekordia:fertilizerInRange(dt)
end;


function Rekordia:delete()
	removeTrigger(self.trigger);
    removeDeleteListener(self.trigger, self.deleteListenerId);
end;

function Rekordia:readStream(streamId, connection)
	self.fill = streamReadBool(streamId);
end;

function Rekordia:writeStream(streamId, connection)
	streamWriteBool(streamId, self.fill);
end;


function Rekordia:mouseEvent(posX, posY, isDown, isUp, button)
end;


function Rekordia:keyEvent(unicode, sym, modifier, isDown)

end;


function Rekordia:draw()

	
end;

function Rekordia:setIsWaterTankFilling(superFunc, isFilling, noEventSend)
    -- Do your logic here
    print("Cur filling: "..tostring(self.isWaterTankFilling));
    print("New filling: "..tostring(isFilling));

    -- also execute the original code
    superFunc(self, isFilling, noEventSend);
end;

function Rekordia:onAttach()
end; 	


function Rekordia:onDetach()
end;

function Rekordia:manureTriggerCallback(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
    if onEnter then
		if otherId ~= nil then
			local trailer = g_currentMission.objectToTrailer[otherId];
			if trailer ~= nil then
				if trailer.fillTypes ~= nil and trailer.setFillLevel ~= nil and trailer.fillLevel ~= nil and trailer.rootNode ~= self.rootNode and trailer:allowFillType(self.currentFillType, true) then
					self.trailerInTrigger = trailer;
				end;
			end;
		end;		
	elseif onLeave then
		if otherId ~= nil then		
			local trailer = g_currentMission.objectToTrailer[otherId];
			if trailer ~= nil then
				if self.trailerInTrigger == trailer then
					self.trailerInTrigger = nil;
				end;
			end;
		end;
	end;
end;
------------------------------------------------------------ EVENT ------------------------------------------------------------
RekordiaEvent = {};
RekordiaEvent_mt = Class(RekordiaEvent, Event);

InitEventClass(RekordiaEvent, "RekordiaEvent");

function RekordiaEvent:emptyNew()
    local self = Event:new(RekordiaEvent_mt);
    self.className="RekordiaEvent";
    return self;
end;

function RekordiaEvent:new(object, data)
    local self = RekordiaEvent:emptyNew()
    self.object = object;
	self.data = data;
    return self;
end;

function RekordiaEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.object = networkGetObject(id);
	self.data = streamReadBool(streamId);
	self.object.fill = self.data;
	if not connection:getIsServer() then
        g_server:broadcastEvent(RekordiaEvent:new(self.object, self.data), nil, connection, self.object);
    end;
end;

function RekordiaEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.object));
	streamWriteBool(streamId, self.data);
end;

function RekordiaEvent.sendEvent(vehicle, data)
	if g_server ~= nil then
		g_server:broadcastEvent(RekordiaEvent:new(vehicle, data), nil, nil, vehicle);
	else
		g_client:getServerConnection():sendEvent(RekordiaEvent:new(vehicle, data));
	end;
end;